"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = exports.coerceUserCreatePullRequestOptions = exports.CREATE_PR_COMMAND = void 0;
const rest_1 = require("@octokit/rest");
const git = require("./handle-git-dir-change");
const __1 = require("../");
const logger_1 = require("../logger");
const yargs = require("yargs");
exports.CREATE_PR_COMMAND = 'pr';
/**
 * map yargs to user pull request otions
 */
function coerceUserCreatePullRequestOptions() {
    return {
        upstreamRepo: yargs.argv.upstreamRepo,
        upstreamOwner: yargs.argv.upstreamOwner,
        message: yargs.argv.message,
        description: yargs.argv.description,
        title: yargs.argv.title,
        branch: yargs.argv.branch,
        force: yargs.argv.force,
        primary: yargs.argv.primary,
        maintainersCanModify: yargs.argv.maintainersCanModify,
        fork: yargs.argv.fork,
    };
}
exports.coerceUserCreatePullRequestOptions = coerceUserCreatePullRequestOptions;
/**
 * main workflow entrance
 */
async function main() {
    try {
        logger_1.setupLogger();
        const options = coerceUserCreatePullRequestOptions();
        if (!process.env.ACCESS_TOKEN) {
            throw Error('The ACCESS_TOKEN should not be undefined');
        }
        const octokit = new rest_1.Octokit({ auth: process.env.ACCESS_TOKEN });
        let changes;
        switch (yargs.argv._[0]) {
            case exports.CREATE_PR_COMMAND:
                changes = await git.getChanges(yargs.argv['git-dir']);
                await __1.createPullRequest(octokit, changes, options, logger_1.logger);
                break;
            default:
                // yargs should have caught this.
                throw Error(`Unhandled command detected: ${yargs.argv._[0]}`);
        }
    }
    catch (err) {
        logger_1.logger.error('Workflow failed');
        throw err;
    }
}
exports.main = main;
//# sourceMappingURL=workflow.js.map